/* ==================================================================

  File: RS232.cpp

  Description:

  This is the windows specific serial port routines 
  
  Version: 1.4    Compiled for Visual Studio 2013 Express    
  =================================================================*/

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include "dacio.h"
#include<TCHAR.H>

static HANDLE comHandle;  // file descriptor


/* Open port at 9600 baud, 8 data bits and no parity */
//int SerialOpenPort(char *port)
int SerialOpenPort(TCHAR *port)
{
	DCB dcb;
	COMMTIMEOUTS commtimeouts;
	DWORD result;
	
	/* Open a windows handle to the COM port */
	comHandle=CreateFile(port,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	
	if (comHandle) 
	{
	   SetupComm(comHandle,128,128);
	   /* Setup com port timeouts */
	   commtimeouts.ReadIntervalTimeout=0;
	   commtimeouts.ReadTotalTimeoutMultiplier=100;
	   commtimeouts.ReadTotalTimeoutConstant=0;
	   commtimeouts.WriteTotalTimeoutMultiplier=0;
	   commtimeouts.WriteTotalTimeoutConstant=100;
	   SetCommTimeouts(comHandle,&commtimeouts);
	   
	   /* Get existing com port settings and modify*/
	   dcb.DCBlength=sizeof(dcb);
	   GetCommState(comHandle,&dcb);
	   result=BuildCommDCB((TCHAR *)"baud=9600 parity=N data=8 stop=1",&dcb);
	   dcb.fBinary=TRUE;
	   dcb.fDtrControl=DTR_CONTROL_DISABLE;
	   dcb.fOutxDsrFlow=FALSE;
	   dcb.fNull=FALSE;
	   dcb.fRtsControl=RTS_CONTROL_ENABLE;
	   dcb.fOutxCtsFlow=FALSE;
	   dcb.fDsrSensitivity=FALSE;
	   dcb.fOutX=FALSE;
	   dcb.fInX=FALSE;

     /* Now set com port settings*/
     result=SetCommState(comHandle,&dcb);
	   if (!result)
	   {
		   /* some error - was unable to set com port settings correctly */
		   CloseHandle(comHandle);
		   comHandle=NULL;
	   }
	}
	return comHandle!=NULL?0:-1;
}

/* Open port at 19200 baud, 8 data bits and no parity */
int SerialOpenPortBad(TCHAR *port)
{
	DCB dcb;
	COMMTIMEOUTS commtimeouts;
	DWORD result;

	/* Open a windows handle to the COM port */
	comHandle=CreateFile(port,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);

	if (comHandle)
	{
	   SetupComm(comHandle,128,128);
	   /* Setup com port timeouts */
	   commtimeouts.ReadIntervalTimeout=0;
	   commtimeouts.ReadTotalTimeoutMultiplier=100;
	   commtimeouts.ReadTotalTimeoutConstant=0;
	   commtimeouts.WriteTotalTimeoutMultiplier=0;
	   commtimeouts.WriteTotalTimeoutConstant=100;
	   SetCommTimeouts(comHandle,&commtimeouts);

	   /* Get existing com port settings and modify*/
	   dcb.DCBlength=sizeof(dcb);
	   GetCommState(comHandle,&dcb);
	   result=BuildCommDCB((TCHAR *)"baud=19200 parity=N data=8 stop=1",&dcb);
	   dcb.fBinary=TRUE;
	   dcb.fDtrControl=DTR_CONTROL_DISABLE;
	   dcb.fOutxDsrFlow=FALSE;
	   dcb.fNull=FALSE;
	   dcb.fRtsControl=RTS_CONTROL_ENABLE;
	   dcb.fOutxCtsFlow=FALSE;
	   dcb.fDsrSensitivity=FALSE;
	   dcb.fOutX=FALSE;
	   dcb.fInX=FALSE;

     /* Now set com port settings*/
     result=SetCommState(comHandle,&dcb);
	   if (!result)
	   {
		   /* some error - was unable to set com port settings correctly */
		   CloseHandle(comHandle);
		   comHandle=NULL;
	   }
	}
	return comHandle!=NULL?0:-1;
}

/* Open port at 115200 baud, 8 data bits and no parity */
int SerialOpenPortFast(TCHAR  *port)
{
	DCB dcb;
	COMMTIMEOUTS commtimeouts;
	DWORD result;

	/* Open a windows handle to the COM port */
	comHandle=CreateFile(port,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);

	if (comHandle)
	{
	   SetupComm(comHandle,128,128);
	   /* Setup com port timeouts */
	   commtimeouts.ReadIntervalTimeout=0;
	   commtimeouts.ReadTotalTimeoutMultiplier=100;
	   commtimeouts.ReadTotalTimeoutConstant=0;
	   commtimeouts.WriteTotalTimeoutMultiplier=0;
	   commtimeouts.WriteTotalTimeoutConstant=100;
	   SetCommTimeouts(comHandle,&commtimeouts);

	   /* Get existing com port settings and modify*/
	   dcb.DCBlength=sizeof(dcb);
	   GetCommState(comHandle,&dcb);
	   result=BuildCommDCB((TCHAR *)"baud=115200 parity=N data=8 stop=1",&dcb);
	   dcb.fBinary=TRUE;
	   dcb.fDtrControl=DTR_CONTROL_DISABLE;
	   dcb.fOutxDsrFlow=FALSE;
	   dcb.fNull=FALSE;
	   dcb.fRtsControl=RTS_CONTROL_ENABLE;
	   dcb.fOutxCtsFlow=FALSE;
	   dcb.fDsrSensitivity=FALSE;
	   dcb.fOutX=FALSE;
	   dcb.fInX=FALSE;

     /* Now set com port settings*/
     result=SetCommState(comHandle,&dcb);
	   if (!result)
	   {
		   /* some error - was unable to set com port settings correctly */
		   CloseHandle(comHandle);
		   comHandle=NULL;
	   }
	}
	return comHandle!=NULL?0:-1;
}

/* Close the port when finished */
void SerialClosePort()
{
  CloseHandle(comHandle);
}

/* Write a byte to the DACIO */
int SerialWriteCh(int byte)
{
  char c=(char)byte;
  DWORD n=0;
  WriteFile(comHandle,&c,1,&n,NULL);
  return n==1;
}

/* Read a byte from the DACIO */
unsigned int SerialReadCh(void)
{
  char c;
  DWORD n;
  unsigned int tmp;
  ReadFile(comHandle,&c,1,&n,NULL);
  tmp=((unsigned int)c) & 0xFF;;  
  return (n==1)?tmp:-1;
}




