/*===========================================================================

  File: DACIO.cpp
    
  Description:

  This file contains the general C language DACIO routines and example demo

  This code example supports the DACIO I/O boards.
   
  As shipped this code is meant to compile under Windows.
  However, if you port the following functions the code 
  should be easy enough to convert to most standard C 
  compilers running under a modern OS:
   
  SerialOpenPort  - Open a com port (returns -1 for error)
  SerialClosePort - Close the com port 
  SerialWriteCh   - Write a byte to the COM port (returns 1 for success)
  SerialReadCh    - Read a byte from the COM port (returns byte or -1 for error)
  
  Version: 1.4    Compiled for Visual Studio 2013 Express    

  ===========================================================================*/
// dacio.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "dacio.h"
#include <windows.h>  // mainly for Beep, atoi and Sleep function (not required otherwise).

/* Example demo program */
int _tmain(int argc, _TCHAR* argv[])
{
  int carry_on = 'y';
  if (argc<2)
  {
      fprintf(stderr,"Usage: dacio.exe port e.g. dacio.exe COM4\n");
      return 1;
  }

  int x;

  while(carry_on == 'y' || carry_on == 'Y' || carry_on == 10)
  {
     if(SerialOpenPortFast((TCHAR*)argv[1])<0)
     {
	  printf("\nError opening COM port: %s", argv[1]);
         return 1;
     }

     printf("\n====================================\n");

     int MID = Get_ModuleID();
     printf("\nModule ID: %d\n", MID);

     int MVER = Get_ModuleSoftwareVersion();
     printf("Module Software Versions: %d\n", MVER);

      if(SetDigitalPORT('B', 0) == 0)
            printf("Setting PORTB I/O direction as all outputs\n");
      else
            printf("Failed to set I/O direction to PORTB\n");
      if(SetDigitalPORT('C', 0) == 0)
            printf("Setting PORTB I/O direction as all outputs\n");
      else
            printf("Failed to set I/O direction to PORTB\n");


      if(WriteDigitalPORT('B', 170) == 0);
      else printf("\nFailed to write 170 to PORTB\n");
      short_delay();

      if(WriteDigitalPORT('C', 170) == 0);
      else printf("\nFailed to write 170 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 170)
         printf("\nError reading PORTB = %d. Should be 170\n", x);

      x = ReadDigitalPORT('C');
      if(x != 170)
         printf("\nError reading PORTC = %d. Should be 170\n", x);

      if(WriteDigitalPORT('B', 49) == 0);
      else printf("\nFailed to write 49 to PORTB\n");
      short_delay();

      if(WriteDigitalPORT('C', 49) == 0);
      else printf("\nFailed to write 49 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 49)
         printf("\nError reading PORTB = %d. Should be 49\n", x);

      x = ReadDigitalPORT('C');
      if(x != 49)
         printf("\nError reading PORTC = %d. Should be 49\n", x);


      //==========================================================
      if(WriteDigitalPORT('B', 85) == 0);
      else printf("\nFailed to write 85 to PORTB\n");

      short_delay();

      if(WriteDigitalPORT('C', 85) == 0);
      else printf("\nFailed to write 85 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 85)
      {
         printf("\nError reading PORTB = %d. Should be 85\n", x);
         Beep(400,1000);

      }
      x = ReadDigitalPORT('C');
      if(x != 85)
      {
         printf("\nError reading PORTC = %d. Should be 85\n", x);
         Beep(400,1000);
      }

      short_delay();


      x = Get_PORTA_Value(0);
      printf("\nAnalogue Channel A0 = %d\n", x);
      printf("\nPress To Test RED LED - Red LED \n", x);
      getchar();


      Beep(700,200);
      Sleep(100);
      Beep(700,200);
      Set_RED_LED(1); // Turn RED LED on.
      printf("\nPress ENTER to test again or 'q' to quit \n");
      carry_on = getchar();
      //clrscr();
	  printf("\n\n===============================\n\n");
      Set_RED_LED(0); // Turn RED LED off.
      printf("Closing port...\n");      
      SerialClosePort();
  }
  printf("Closing port...\n");
  SerialClosePort();
  getchar();
  return 0;
}

/* =======================================================================
                         DACIO specific functions 

   =====================================================================*/

int WriteDigitalPORT(char portch, int value)
{
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf_s(temp_str, "!%c=%0.3d;", portch, value);   // e.g. "!B=255;" 
  SerialWriteString(temp_str);
  
  temp_str[0] = SerialReadCh();  /* should be '!" */
  temp_str[1] = SerialReadCh();  /* should be <CR> */
  if(temp_str[0] == '!' && temp_str[1] == 0x0D) return 0;
  else return -1;
}

int SetDigitalPORT(char portch, int value)
{
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf_s(temp_str, "!S%c=%0.3d;", portch, value);   // e.g. "!SB=255;" 
  SerialWriteString(temp_str);
  
  temp_str[0] = SerialReadCh();  /* should be '!" */
  temp_str[1] = SerialReadCh();  /* should be <CR> */
  if(temp_str[0] == '!' && temp_str[1] == 0x0D) return 0;
  else return -1;
}

int ReadDigitalPORT(char portch)
{
  int result;
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf_s(temp_str, "!%c?;", portch);   // e.g. "!B?;" 
  SerialWriteString(temp_str);

  temp_str[0] = SerialReadCh();  
  if(temp_str[0] == '!')             // got a good reply '!'.  
  {
  	 int i;
	 for(i = 0; i < 4; i++)
   	 {
   	 	    temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	 } 
	 temp_str[i] = 0;               // discard terminating (CR) char 
     result = atoi(temp_str); // convert ASCII digits from unit into an integer
  }
  else
  {
      SerialReadCh();  // read '?';
   	  result = -1;
  }
  return result;
}

int Get_PORTA_Value(int ch)
{
  char temp_str[10];
  int result;
  SerialWriteCh('!'); SerialWriteCh('A'); SerialWriteCh(ch+'0'); SerialWriteCh('?'); SerialWriteCh(';');
  
  temp_str[0] = SerialReadCh();  
  if(temp_str[0] == '!')             // got a good reply '!'.  
  {
   	  int i;
	  for(i = 0; i < 4; i++)
   	     temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	  temp_str[i] = 0;               // discard terminating (CR) char 
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}

int Get_ModuleID(void)
{
  char temp_str[10];
  int result;
  // send "!SMID?;"
  SerialWriteString("!SMID?;");

  temp_str[0] = SerialReadCh();
  if(temp_str[0] == '!')             // got a good reply '!'.
  {
   	  int i;
	  for(i = 0; i < 4; i++)
   	     temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	  temp_str[i] = 0;                  // discard terminating (CR) char
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}

int Get_ModuleSoftwareVersion(void)
{
  char temp_str[10];
  int result;
  // send "!SVER?;"
  SerialWriteString("!SVER?;");

  temp_str[0] = SerialReadCh();
  if(temp_str[0] == '!')             // got a good reply '!'.
  {
   	  int i;
	  for(i = 0; i < 3; i++)
   	     temp_str[i] = SerialReadCh();  // get the 2 digits and get terminating (CR) char
	  temp_str[i] = 0;                  // discard terminating (CR) char
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}

int Set_RED_LED(int state)
{
	char temp_str[10];
	// send "!XLED1=1;" to turn ON LED and  "!XLED1=0;" to turn OFF LED;
	SerialWriteString("!XLED1=");
	if (state == 1)
		SerialWriteCh('1');
	else if (state == 0)
		SerialWriteCh('0');
	SerialWriteCh(';');
	temp_str[0] = SerialReadCh();  /* should be '!" */
	temp_str[1] = SerialReadCh();  /* should be <CR> */
	if (temp_str[0] == '!' && temp_str[1] == 0x0D) return 0;
	else return -1;
}


/* =======================================================================
                         Auxilliary functions 

   =====================================================================*/

void SerialWriteString(char *s)
{
	 while(*s)
	 {
        SerialWriteCh(*s);
        s++;
	 } 
}	

void pause()
{
  printf("Press ENTER\n");
  getchar();
}

void short_delay()
{
  Sleep(200);
}




